import 'package:fis_measure/interfaces/process/calculators/output.dart';
import 'package:fis_measure/interfaces/process/calculators/values.dart';
import 'package:fis_measure/process/calcuators/formulas/general.dart';
import 'package:fis_measure/process/items/item.dart';
import 'package:fis_measure/process/primitives/combos/two_area.dart';
import 'package:fis_measure/process/primitives/combos/two_length.dart';
import 'package:fis_measure/process/primitives/trace.dart';

import 'calculator.dart';

class ABRatioCal extends Calculator<TwoLengthAbstract, double> {
  ABRatioCal(TwoLengthAbstract ref) : super(ref);

  @override
  void calculate() {
    if (ref.feature == null) return;

    final a1 = _pickChildValue(ref.child1);
    final a2 = _pickChildValue(ref.child2);

    final feature = ref.feature!;
    final viewport = feature.hostVisualArea!.viewport!;

    if (a1 != null && a2 != null) {
      final value = ratioCal(
        a1,
        a2,
      );
      updateFloatValue(value);
    }
  }

  static double ratioCal(double d1, double d2) {
    return d1 / d2;
  }

  double? _pickChildValue(MeasureItem item) {
    if (item.calculator == null) return null;
    ValueBase? value;
    if (item.measuredFeatures.isNotEmpty) {
      value = item.measuredFeatures.first.value;
    } else if (item.feature != null) {
      value = item.feature!.value;
    }
    if (value != null) {
      return (value as FloatValue).value ?? 0;
    }
    return null;
  }
}
