import 'dart:ui';

import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/types.dart';
import 'package:fis_measure/interfaces/process/workspace/point_info.dart';
import 'package:fis_measure/process/calcuators/three_distance.dart';
import 'package:fis_measure/process/items/item_feature.dart';
import 'package:fis_measure/process/items/top_item.dart';
import 'package:fis_measure/process/items/top_item_feature.dart';
import 'package:fis_measure/process/primitives/straightline.dart';

class LWHStraightLine extends TopMeasureItem<LWHStraightlineFeature> {
  static const String _lineLKey = "L";
  static const String _lineWKey = "W";
  static const String _lineHKey = "H";

  late final StraightLine l;
  late final StraightLine w;
  late final StraightLine h;

  LWHStraightLine(ItemMeta meta) : super(meta) {
    final metaL = meta.getChildByName(_lineLKey)!;
    final metaH = meta.getChildByName(_lineHKey)!;
    final metaW = meta.getChildByName(_lineWKey)!;
    l = StraightLine.createDistance(metaL, this);
    w = StraightLine.createDistance(metaW, this);
    h = StraightLine.createDistance(metaH, this);
    childItems.add(l);
    childItems.add(w);
    childItems.add(h);
  }

  @override
  bool get finishAfterUnactive => true;

  @override
  LWHStraightlineFeature buildFeature() => LWHStraightlineFeature(this);

  @override
  void onCancelingOnce() {}

  //ThreeDistMeanCalculator

  static LWHStraightLine createVolume(ItemMeta meta, [IMeasureItem? parent]) {
    if (meta.measureType != MeasureTypes.VolumeThreeDistance) {
      throw ArgumentError();
    }
    var lwh = LWHStraightLine(meta);
    lwh.calculator = VolumeThreeDistanceCal(lwh);

    return lwh;
  }

  static LWHStraightLine createMean(ItemMeta meta, [IMeasureItem? parent]) {
    if (meta.measureType != MeasureTypes.ThreeDistanceMean) {
      throw ArgumentError();
    }
    var lwh = LWHStraightLine(meta);
    lwh.calculator = ThreeDistMeanCalculator(lwh);
    return lwh;
  }

  static LWHStraightLine createMax(ItemMeta meta, [IMeasureItem? parent]) {
    var lwh = LWHStraightLine(meta);
    lwh.calculator = ThreeDistMaxCalculator(lwh);
    return lwh;
  }
}

class LWHStraightlineFeature extends TopMeasureItemFeature {
  LWHStraightlineFeature(
    ITopMeasureItem refItem,
  ) : super(refItem);
}
