import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/index.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/view/ai_result_modifier/view.dart';
import 'package:fis_measure/view/paint/ai_patint_controller.dart';
import 'package:fis_measure/view/paint/date_structure.dart';
import 'package:fis_measure/view/paint/parts/ai_resul_info.dart';
import 'package:fis_measure/view/paint/parts/feature_analysis.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vid/us/vid_us_image.dart';
import 'package:fis_ui/index.dart';

class AIResultPanel extends StatefulWidget {
  final List<AIDetectedObject> aiDetectedObject;

  /// ai部位
  final DiagnosisOrganEnum diagnosisOrgan;

  const AIResultPanel(
    this.aiDetectedObject,
    this.diagnosisOrgan, {
    Key? key,
  }) : super(key: key);

  @override
  State<AIResultPanel> createState() => _AIResultPanelState();
}

class _AIResultPanelState extends State<AIResultPanel> {
  late AIDetectedObject aiDetectedObjectItem;
  final aiPatintController = Get.find<AiPatintController>();
  MeasureDataController get measureData => Get.find<MeasureDataController>();
  VidPlayerController get playerController =>
      Get.find<IPlayerController>() as VidPlayerController;
  String get remedicalCode => measureData.measureImageData.remedicalCode ?? "";
  int get currFrameIndex => playerController.currentFrameIndex;
  VidUsImage get currFrame => playerController.currentFrame!;

  ///屏幕缩放比例
  double get devicePixelRatio => MediaQuery.of(context).devicePixelRatio;
  @override
  Widget build(BuildContext context) {
    double verticalSpacing = 10;
    if (devicePixelRatio > 1.25) {
      verticalSpacing = 2;
    } else if (devicePixelRatio > 1) {
      verticalSpacing = 5;
    }
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        Expanded(child: Container()),
        SizedBox(
          /// TODO:[Gavin] 多语言-UI-整合
          width: i18nBook.isCurrentChinese ? 210 : 270,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              if (widget.aiDetectedObject.length > 1)
                Container(
                  child: Wrap(
                    spacing: 2.0, // 主轴(水平)方向间距
                    runSpacing: 2.0, // 纵轴（垂直）方向间距
                    direction: Axis.horizontal,
                    alignment: WrapAlignment.start,
                    children: List<Widget>.generate(
                      widget.aiDetectedObject.length,
                      (index) {
                        return GestureDetector(
                          onTap: () {
                            aiPatintController.state.aiResultIndex = index;
                          },
                          child: Obx(
                            () => Container(
                              decoration: BoxDecoration(
                                color: aiPatintController.state.aiResultIndex ==
                                        index
                                    ? const Color.fromRGBO(54, 169, 206, 1)
                                    : Colors.grey,
                              ),
                              width: 38 / devicePixelRatio,
                              height: 35 / devicePixelRatio,
                              child: Center(
                                child: Text(
                                  '${index + 1}',
                                  style: const TextStyle(
                                    color: Colors.white,
                                  ),
                                ),
                              ),
                            ),
                          ),
                        );
                      },
                    ),
                  ),
                  color: Colors.transparent,
                ),
              SizedBox(
                height: verticalSpacing,
              ),
              Obx(() {
                try {
                  aiDetectedObjectItem = widget
                      .aiDetectedObject[aiPatintController.state.aiResultIndex];
                  if (aiDetectedObjectItem.label != 0) {
                    return Column(
                      children: [
                        ResultInfo(
                          widget.aiDetectedObject,
                        ),
                        SizedBox(
                          height: verticalSpacing,
                        ),
                        aiDetectedObjectItem.descriptions?.isNotEmpty ?? false
                            ? FeatureAnalysis(
                                aiDetectedObjectItem.descriptions,
                              )
                            : const SizedBox(),
                        SizedBox(
                          height: verticalSpacing,
                        ),
                        if (_canEditAI())
                          InkWell(
                            onTap: () {
                              Get.dialog(
                                AiResultModifierDialog(
                                  remedicalCode: remedicalCode,
                                  currFrameIndex: currFrameIndex,
                                  currFrame: currFrame,
                                  currentLabel: _getCurrentAIIndex(),
                                ),
                              );
                            },
                            child: Text(
                              i18nBook.measure.editAiResult.t,
                              style: const TextStyle(
                                color: Colors.white,
                                decoration: TextDecoration.underline,
                              ),
                            ),
                          ),
                      ],
                    );
                  } else {
                    return const SizedBox();
                  }
                } catch (e) {
                  return const SizedBox();
                }
              })
            ],
          ),
        ),
      ],
    );
  }

  int _getCurrentAIIndex() {
    var selectedAIResultIndex = aiPatintController.state.aiResultIndex;
    var indexList = <int>[];
    widget.aiDetectedObject.forEach((element) {
      var index = widget.aiDetectedObject.indexOf(element);
      if (element.descriptions?.isNotEmpty ?? false) {
        indexList.add(index);
      }
    });
    return indexList.indexOf(selectedAIResultIndex);
  }

  bool _canEditAI() {
    if (kIsMobile ||
        widget.diagnosisOrgan == DiagnosisOrganEnum.CarotidArtery) {
      return false;
    }
    return aiDetectedObjectItem.descriptions?.isNotEmpty ?? false;
  }
}
